import os
import unittest
import hashlib
import numpy as np
from sysconfig import get_platform

from G722 import G722

IS_S390X = get_platform() == 'linux-s390x'

@unittest.skipIf(IS_S390X, "G722 encode/decode test is broken on BE; skipping until fixed")
class TestEncodeDecode(unittest.TestCase):
    DATA_DIR = os.path.join(os.path.dirname(__file__), '../test_data')
    PCM_FILE = os.path.join(DATA_DIR, 'pcminb.dat')
    MD5_FILE = os.path.join(DATA_DIR, 'pcminb.checksum')

    def setUp(self):
        # load raw PCM as int16 array
        with open(self.PCM_FILE, 'rb') as f:
            pcm_bytes = f.read()
        self.pcm = np.frombuffer(pcm_bytes, dtype='<i2')
        total = int(self.pcm.sum(dtype=np.int64))
        self.assertEqual(total, 7206958, "Input checksum fail")

        # load expected MD5s; expect lines like "48000_8000 2334 cfa9e73241616fa366293aafdf2fde32 11a9661b59beeae7d520c38e1a3524e4"
        self.expected = {}
        with open(self.MD5_FILE, 'r') as f:
            for line in (l for l in f if not l.startswith('#')):
                key, size, denc, ddec = line.strip().split()
                self.expected[key] = (denc, ddec, int(size))

    def test_all_bitrates_and_sample_rates(self):
        bitrates    = (48000, 56000, 64000)
        sample_rates = (8000, 16000)

        for br in bitrates:
            for sr in sample_rates:
                key = f"{br}_{sr}"
                with self.subTest(bitrate=br, sample_rate=sr):
                    codec = G722(sr, br)
                    encoded = codec.encode(self.pcm)
                    md5 = hashlib.md5(encoded).hexdigest()

                    # sanity check that we have an expected hash
                    self.assertIn(key, self.expected, f"no MD5 entry for {key}")
                    denc, ddec, csize = self.expected[key]
                    self.assertEqual(csize, len(encoded),
                                     f"Encoded size mismatch for {key}")
                    self.assertEqual(denc, md5,
                                     f"MD5 mismatch for {key}")

                    decoded = codec.decode(encoded)
                    decoded_le = decoded.astype('<i2')
                    decoded_bytes = decoded_le.tobytes()
                    self.assertEqual(len(decoded), len(self.pcm),
                                     f"Decoded size mismatch for {key}")
                    md5_dec  = hashlib.md5(decoded_bytes).hexdigest()
                    self.assertEqual(ddec, md5_dec,
                                     f"Decoded MD5 mismatch for {key}")

if __name__ == '__main__':
    unittest.main()



import unittest

from G722 import G722

class TestDecoder(unittest.TestCase):
    def test_decode_len(self):
        bitrates = [48000, 56000, 64000]
        sample_rates = [8000, 16000]

        for bitrate in bitrates:
            for sample_rate in sample_rates:
                with self.subTest(bitrate=bitrate, sample_rate=sample_rate):
                    g722 = G722(sample_rate, bitrate)
                    encoded_data = b"0123"  # Example encoded data
                    decoded_data = g722.decode(encoded_data)
                    got = len(decoded_data)
                    want = 4 if sample_rate == 8000 else 8
                    self.assertEqual(want, got)


if __name__ == '__main__':
    unittest.main()



import unittest

import numpy as np

from G722 import G722

class TestEncoder(unittest.TestCase):
    def test_encode_len(self):
        bitrates = (48000, 56000, 64000)
        sample_rates = (8000, 16000)
        sample_inputs = (b"\x00\x01\x02\x03", np.array((0, 1, 2, 3), dtype=np.int16), (0, 1, 2, 3), ('aa', 'bb', 'cc'), 42)
        def do_test(br, sr, id):
            with self.subTest(bitrate=br, sample_rate=sr, input_data=id):
                g722 = G722(sr, br)
                try:
                    encoded_data = g722.encode(id)
                except TypeError:
                    got = isinstance(id, int) or isinstance(id[0], str)
                    want = True
                    self.assertEqual(want, got)
                    return
                else:
                    got = isinstance(id, int)
                    want = False
                    self.assertEqual(want, got)
                got = len(encoded_data)
                want = 4 if sample_rate == 8000 else 2
                self.assertEqual(want, got)
        for bitrate in bitrates:
            for sample_rate in sample_rates:
                for sample_input in sample_inputs:
                    do_test(bitrate, sample_rate, sample_input)

if __name__ == '__main__':
    unittest.main()



import sys
from distutils.core import Command

class CheckVersion(Command):
    description = "Check version number"
    user_options = [
        ('tag=', 't', 'git tag to compare against package version'),
    ]
    extra_compile_args = []
    extra_link_args = []

    def initialize_options(self):
        self.tag = None

    def finalize_options(self):
        if not self.tag:
            raise DistutilsOptionError("You must specify --tag")

    def run(self):
        pkg_version = self.distribution.get_version()
        if self.tag == f'v{pkg_version}':
            return
        sys.stderr.write(f"❌ version {pkg_version} != tag {self.tag}\n")
        sys.exit(1)



#!/usr/bin/env python

from sys import exit, argv
from distutils.core import setup, Extension
from os.path import exists, realpath, dirname, join as path_join
from sys import argv as sys_argv, path as sys_path

from sysconfig import get_platform

sys_path.insert(0, realpath(dirname(__file__)))
from build_tools.CheckVersion import CheckVersion

def main():

    mod_name = 'G722'
    mod_name_dbg = mod_name + '_debug'

    mod_dir = dirname(realpath(sys_argv[0]))
    src_dir = './' if exists('g722_decode.c') else '../'
    mod_fname = mod_name + '_mod.c'
    mod_dir = '' if exists(mod_fname) else 'python/'

    def np_include():
        import numpy as np
        return f'-I{np.get_include()}'

    is_win = get_platform().startswith('win')
    is_mac = get_platform().startswith('macosx-')

    compile_args = [f'-I{src_dir}', np_include()]
    if not is_win:
        compile_args.append('-flto')
    link_args = ['-flto',] if not is_win else []
    if not is_mac and not is_win:
        smap_fname = f'{mod_dir}symbols.map'
        link_args.append(f'-Wl,--version-script={smap_fname}')
    debug_cflags = ['-g3', '-O0', '-DDEBUG_MOD']
    debug_link_args = ['-g3', '-O0']
    mod_common_args = {
        'sources': [mod_dir + mod_fname, src_dir + 'g722_decode.c', src_dir + 'g722_encode.c'],
        'extra_compile_args': compile_args,
        'extra_link_args': link_args
    }
    mod_debug_args = mod_common_args.copy()
    mod_debug_args['extra_compile_args'] = mod_debug_args['extra_compile_args'] + debug_cflags
    mod_debug_args['extra_link_args'] = mod_debug_args['extra_link_args'] + debug_link_args

    module1 = Extension(mod_name, **mod_common_args)
    module2 = Extension(mod_name_dbg, **mod_debug_args)

    requirements = [x.strip() for x in open(mod_dir + "requirements.txt", "r").readlines()]
    with open(src_dir + "README.md", "r") as fh:
        long_description = fh.read()

    kwargs = {
        'name':mod_name,
        'version':'1.2.1',
        'description':'This is a package for G.722 module',
        'long_description': long_description,
        'long_description_content_type': "text/markdown",
        'author':'Maksym Sobolyev',
        'author_email':'sobomax@sippysoft.com',
        'url':'https://github.com/sippy/libg722',
        'ext_modules': [module1, module2],
        'install_requires': requirements,
        'cmdclass': {'checkversion': CheckVersion},
        'license': 'Public-Domain',
        'classifiers': [
                'Operating System :: OS Independent',
                'Programming Language :: C',
                'Programming Language :: Python'
        ]
    }

    setup (**kwargs)

if __name__ == '__main__': main()
